// { dg-do compile }

// 2006-02-02  Paolo Carlini  <pcarlini@suse.de>
//
// Copyright (C) 2006 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this library; see the file COPYING.  If not, write to the Free
// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
// USA.

#include <tr1/array>
#include <tr1/cctype>
#include <tr1/cfenv>
#include <tr1/cfloat>
#include <tr1/cinttypes>
#include <tr1/climits>
#include <tr1/cmath>
#include <tr1/complex>
#include <tr1/cstdarg>
#include <tr1/cstdbool>
#include <tr1/cstdint>
#include <tr1/cstdio>
#include <tr1/cstdlib>
#include <tr1/ctgmath>
#include <tr1/ctime>
#include <tr1/ctype.h>
#include <tr1/cwchar>
#include <tr1/cwctype>
#include <tr1/fenv.h>
#include <tr1/float.h>
#include <tr1/functional>
#include <tr1/inttypes.h>
#include <tr1/limits.h>
#include <tr1/math.h>
#include <tr1/memory>
#include <tr1/random>
#include <tr1/stdarg.h>
#include <tr1/stdbool.h>
#include <tr1/stdint.h>
#include <tr1/stdio.h>
#include <tr1/stdlib.h>
#include <tr1/tgmath.h>
#include <tr1/tuple>
#include <tr1/type_traits>
#include <tr1/unordered_set>
#include <tr1/unordered_map>
#include <tr1/utility>
#include <tr1/wchar.h>
#include <tr1/wctype.h>
